export interface LoginFormInputs {
    username: string;
    password: string;
}

// login.viewmodel.ts
import { useAuth } from "@provider/auth.provider";

export default function useLoginViewModel() {
    const { login, loginProperties } = useAuth();
    const { isLoading: loginLoading, error: loginError, reset: resetLogin } = loginProperties;

    const handleLogin = async (data: LoginFormInputs) => {
        resetLogin();
        await login(data.username, data.password);
    };

    const getErrorMessage = (formInvalidError: JSX.Element, backendResponseError: JSX.Element): JSX.Element | null => {
        if (loginError) {
            return backendResponseError;
        }
        return null;
    };

    return {
        handleLogin,
        getErrorMessage,
        loginLoading,
        loginError
    };
}